<?php

namespace XF\Api\Controller;

use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Nodes extends AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('node');
	}

	public function actionGet()
	{
		$nodeTree = $this->getNodeTreeForList();

		/** @var \XF\Mvc\Entity\AbstractCollection $nodes */
		$nodes = $nodeTree->getAllData();

		$result = [
			'tree_map' => $nodeTree->getParentMapSimplified(),
			'nodes' => $nodes->toApiResults()
		];
		return $this->apiResult($result);
	}

	public function actionGetFlattened()
	{
		$nodeTree = $this->getNodeTreeForList();

		$flat = [];
		foreach ($nodeTree->getFlattened() AS $id => $data)
		{
			$flat[$id] = [
				'node' => $data['record']->toApiResult(),
				'depth' => $data['depth']
			];
		}

		return $this->apiResult(['nodes_flat' => $flat]);
	}

	public function actionPost()
	{
		$this->assertAdminPermission('node');
		$this->assertRequiredApiInput(['node_type_id', 'node.title', 'node.parent_node_id']);

		/** @var \XF\Entity\Node $node */
		$node = $this->em()->create('XF:Node');
		$node->node_type_id = $this->filter('node_type_id', 'str');
		if ($node->hasErrors())
		{
			return $this->error($node->getErrors());
		}

		/** @var \XF\Api\ControllerPlugin\Node $nodePlugin */
		$nodePlugin = $this->plugin('XF:Api:Node');
		$nodePlugin->setupNodeSave($node)->run();

		return $this->apiSuccess([
			'node' => $node->toApiResult()
		]);
	}

	protected function getNodeTreeForList()
	{
		$nodeRepo = $this->getNodeRepo();

		if (\XF::isApiCheckingPermissions())
		{
			$nodes = $nodeRepo->getNodeList();
		}
		else
		{
			$nodes = $nodeRepo->getFullNodeListWithTypeData();
		}

		return $nodeRepo->createNodeTree($nodes);
	}

	/**
	 * @return \XF\Repository\Node
	 */
	protected function getNodeRepo()
	{
		return $this->repository('XF:Node');
	}
}